@@ -61,6 +61,8 @@ struct PowerManagerButtonPrivate
 
     UpClient        *upower;
 
+    GDBusProxy      *inhibit_proxy;
+
     /* A list of BatteryDevices  */
     GList           *devices;
 
@@ -320,6 +322,7 @@ power_manager_button_set_label (PowerManagerButton *button, gdouble percentage,
     gtk_label_set_text (GTK_LABEL (button->priv->panel_label), label_string);
 
     g_free (label_string);
+    g_free (remaining_time);
 }
 
 static gboolean
@@ -454,7 +457,14 @@ power_manager_button_update_device_icon_and_details (PowerManagerButton *button,
     details = get_device_description (button->priv->upower, device);
 
     /* If UPower doesn't give us an icon, just use the default */
-    if (icon_name == NULL || g_strcmp0(icon_name, "") == 0)
+    if (g_strcmp0(icon_name, "") == 0)
+    {
+        /* ignore empty icon names */
+        g_free (icon_name);
+        icon_name = NULL;
+    }
+
+    if (icon_name == NULL)
         icon_name = g_strdup (PANEL_DEFAULT_ICON);
 
     pix = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
@@ -483,7 +493,15 @@ power_manager_button_update_device_icon_and_details (PowerManagerButton *button,
         g_object_get (device,
                       "icon-name", &icon_name,
                       NULL);
-        if (icon_name == NULL || g_strcmp0(icon_name, "") == 0)
+
+        /* ignore empty icon names */
+        if (g_strcmp0(icon_name, "") == 0)
+        {
+            g_free (icon_name);
+            icon_name = NULL;
+        }
+
+        if (icon_name == NULL)
             icon_name = g_strdup (PANEL_DEFAULT_ICON_SYMBOLIC);
 #endif
         button->priv->panel_icon_name = g_strdup (icon_name);
@@ -503,7 +521,7 @@ power_manager_button_update_device_icon_and_details (PowerManagerButton *button,
         /* update the image, keep track of the signal ids and the img
          * so we can disconnect it later */
         battery_device->img = gtk_image_new_from_pixbuf (battery_device->pix);
-        g_object_ref (battery_device->img);
+
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
         gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(battery_device->menu_item), battery_device->img);
 G_GNUC_END_IGNORE_DEPRECATIONS
@@ -604,6 +622,8 @@ remove_battery_device (PowerManagerButton *button, BatteryDevice *battery_device
     g_free (battery_device->details);
     g_free (battery_device->object_path);
 
+    battery_device_remove_pix (battery_device);
+
     if (battery_device->device != NULL && UP_IS_DEVICE(battery_device->device))
     {
         /* disconnect the signal handler if we were using it */
@@ -614,6 +634,8 @@ remove_battery_device (PowerManagerButton *button, BatteryDevice *battery_device
         g_object_unref (battery_device->device);
         battery_device->device = NULL;
     }
+
+    g_free (battery_device);
 }
 
 static void
@@ -916,6 +938,23 @@ power_manager_button_init (PowerManagerButton *button)
         button->priv->channel = xfconf_channel_get ("xfce4-power-manager");
     }
 
+    button->priv->inhibit_proxy = g_dbus_proxy_new_sync (g_bus_get_sync (G_BUS_TYPE_SESSION,
+                                                                         NULL,
+                                                                         NULL),
+                                                         G_DBUS_PROXY_FLAGS_NONE,
+                                                         NULL,
+                                                         "org.freedesktop.PowerManagement",
+                                                         "/org/freedesktop/PowerManagement/Inhibit",
+                                                         "org.freedesktop.PowerManagement.Inhibit",
+                                                         NULL,
+                                                         &error);
+
+    if (error != NULL)
+    {
+        g_warning ("error getting inhibit proxy: %s", error->message);
+        g_clear_error (&error);
+    }
+
     /* Sane defaults for the systray and panel icon */
 #ifdef XFCE_PLUGIN
     button->priv->panel_icon_name = g_strdup (PANEL_DEFAULT_ICON_SYMBOLIC);
@@ -1282,6 +1321,83 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     return TRUE;
 }
 
+static void
+display_inhibitors (PowerManagerButton *button, GtkWidget *menu)
+{
+    gboolean needs_seperator = FALSE;
+
+    g_return_if_fail (POWER_MANAGER_IS_BUTTON (button));
+    g_return_if_fail (GTK_IS_MENU (menu));
+
+    if (button->priv->inhibit_proxy)
+    {
+        GVariant *reply;
+        GError   *error = NULL;
+
+        reply = g_dbus_proxy_call_sync (button->priv->inhibit_proxy,
+                                        "GetInhibitors",
+                                        g_variant_new ("()"),
+                                        G_DBUS_CALL_FLAGS_NONE,
+                                        1000,
+                                        NULL,
+                                        &error);
+
+        if (reply != NULL)
+        {
+            GVariantIter *iter;
+            gchar        *value;
+            GtkWidget    *inhibit_mi;
+
+            g_variant_get (reply, "(as)", &iter);
+
+            if (g_variant_iter_n_children (iter) > 0)
+            {
+                needs_seperator = TRUE;
+            }
+
+            /* Add the list of programs to the menu */
+            while (g_variant_iter_next (iter, "s", &value))
+            {
+                GtkWidget *mi, *img;
+
+                /* Translators this is to display which app is inhibiting
+                 * power in the plugin menu. Example:
+                 * VLC is currently inhibiting power management
+                 */
+                gchar *label = g_strdup_printf (_("%s is currently inhibiting power management"), value);
+
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+                mi = gtk_image_menu_item_new_with_label(label);
+G_GNUC_END_IGNORE_DEPRECATIONS
+                /* add the image */
+                img = gtk_image_new_from_icon_name ("gtk-info", GTK_ICON_SIZE_MENU);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+                gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(mi), img);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+                gtk_widget_set_can_focus (mi, FALSE);
+                gtk_widget_show (mi);
+                gtk_menu_shell_append (GTK_MENU_SHELL(button->priv->menu), mi);
+                g_free (label);
+            }
+            g_variant_iter_free (iter);
+            g_variant_unref (reply);
+
+        } else {
+            g_warning ("failed calling GetInhibitors: %s", error->message);
+            g_clear_error (&error);
+        }
+
+        if (needs_seperator)
+        {
+            /* add a separator */
+            GtkWidget * separator_mi = gtk_separator_menu_item_new ();
+            gtk_widget_show (separator_mi);
+            gtk_menu_shell_append (GTK_MENU_SHELL (menu), separator_mi);
+        }
+
+    }
+}
 static void
 decrease_brightness (PowerManagerButton *button)
 {
@@ -1472,6 +1588,9 @@ G_GNUC_END_IGNORE_DEPRECATIONS
                            PROPERTIES_PREFIX PRESENTATION_MODE,
                            G_TYPE_BOOLEAN, G_OBJECT(mi), "active");
 
+    /* Show any applications currently inhibiting now */
+    display_inhibitors (button, menu);
+
     /* Power manager settings */
     mi = gtk_menu_item_new_with_mnemonic (_("_Power manager settings..."));
     gtk_widget_show (mi);
